"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.normalize = void 0;
const generatePostNormalizationErrors_1 = require("./generatePostNormalizationErrors");
const generatePreNormalizationErrors_1 = require("./generatePreNormalizationErrors");
const validate_1 = require("./validate");
function normalize(testLogic, automatonType, finiteAutomaton) {
    (0, validate_1.validate)(finiteAutomaton.label, (0, generatePreNormalizationErrors_1.generatePreNormalizationErrors)(testLogic, automatonType, finiteAutomaton));
    const appendedString = finiteAutomaton.append ?? '';
    if (appendedString) {
        delete finiteAutomaton.append;
    }
    finiteAutomaton.final_states = finiteAutomaton.final_states.map((finalState) => appendedString + finalState.toString());
    finiteAutomaton.alphabets = finiteAutomaton.alphabets.map((alphabet) => alphabet.toString());
    finiteAutomaton.start_state = appendedString + finiteAutomaton.start_state.toString();
    finiteAutomaton.states = finiteAutomaton.states.map((state) => appendedString + state.toString());
    if (finiteAutomaton.epsilon_transitions) {
        Object.entries(finiteAutomaton.epsilon_transitions).forEach(([epsilonTransitionStartState, epsilonTransitionTargetStates]) => {
            finiteAutomaton.epsilon_transitions[epsilonTransitionStartState] =
                epsilonTransitionTargetStates.map((epsilonTransitionTargetState) => epsilonTransitionTargetState.toString());
        });
    }
    function attachToStateRecord(transitionStateRecord, alphabetIndex, state) {
        if (!transitionStateRecord[finiteAutomaton.alphabets[alphabetIndex]]) {
            transitionStateRecord[finiteAutomaton.alphabets[alphabetIndex]] = [state];
        }
        else {
            transitionStateRecord[finiteAutomaton.alphabets[alphabetIndex]].push(state);
        }
    }
    Object.entries(finiteAutomaton.transitions).forEach(([transitionKey, transitionStates]) => {
        const transitionStateRecord = {};
        if (typeof transitionStates !== 'string' && Array.isArray(transitionStates)) {
            transitionStates.forEach((transitionState, transitionStateIndex) => {
                if (transitionState !== null && transitionState !== undefined) {
                    if (Array.isArray(transitionState)) {
                        transitionState.forEach((state) => {
                            attachToStateRecord(transitionStateRecord, transitionStateIndex, appendedString + state.toString());
                        });
                    }
                    else {
                        attachToStateRecord(transitionStateRecord, transitionStateIndex, appendedString + transitionState.toString());
                    }
                }
            });
            finiteAutomaton.transitions[appendedString + transitionKey] =
                transitionStateRecord;
            if (appendedString) {
                delete finiteAutomaton.transitions[transitionKey];
            }
        }
        else if (transitionStates === 'loop') {
            finiteAutomaton.alphabets.forEach((_, alphabetIndex) => {
                attachToStateRecord(transitionStateRecord, alphabetIndex, appendedString + transitionKey.toString());
            });
            finiteAutomaton.transitions[appendedString + transitionKey] =
                transitionStateRecord;
            if (appendedString) {
                delete finiteAutomaton.transitions[transitionKey];
            }
        }
    });
    (0, validate_1.validate)(finiteAutomaton.label, (0, generatePostNormalizationErrors_1.generatePostNormalizationErrors)(finiteAutomaton));
    return finiteAutomaton;
}
exports.normalize = normalize;
