"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertToRegularNfa = void 0;
const epsilonClosureOfState_1 = require("./epsilonClosureOfState");
function convertToRegularNfa(automaton) {
    const { transitions, states, alphabets, epsilon_transitions: epsilonTransitions } = automaton;
    const epsilonClosureOfStateCache = {};
    alphabets.forEach((alphabet) => {
        states.forEach((state) => {
            if (epsilonTransitions && epsilonTransitions[state]) {
                const firstPhaseStates = !epsilonClosureOfStateCache[state]
                    ? (0, epsilonClosureOfState_1.epsilonClosureOfState)(automaton.epsilon_transitions, state)
                    : epsilonClosureOfStateCache[state];
                if (!epsilonClosureOfStateCache[state]) {
                    epsilonClosureOfStateCache[state] = firstPhaseStates;
                }
                const secondPhaseStates = new Set();
                const thirdPhaseSet = new Set();
                firstPhaseStates.forEach((firstPhaseState) => {
                    if (transitions[firstPhaseState]) {
                        transitions[firstPhaseState][alphabet]?.forEach((transitionRecordState) => {
                            secondPhaseStates.add(transitionRecordState);
                        });
                    }
                });
                secondPhaseStates.forEach((secondPhaseState) => {
                    const epsilonClosuredStates = !epsilonClosureOfStateCache[secondPhaseState]
                        ? (0, epsilonClosureOfState_1.epsilonClosureOfState)(automaton.epsilon_transitions, secondPhaseState)
                        : epsilonClosureOfStateCache[secondPhaseState];
                    if (!epsilonClosureOfStateCache[secondPhaseState]) {
                        epsilonClosureOfStateCache[secondPhaseState] = epsilonClosuredStates;
                    }
                    epsilonClosuredStates.forEach((closuredState) => {
                        thirdPhaseSet.add(closuredState);
                    });
                });
                if (transitions[state]) {
                    transitions[state][alphabet] = Array.from(thirdPhaseSet);
                }
                else {
                    transitions[state] = {
                        [alphabet]: Array.from(thirdPhaseSet),
                    };
                }
            }
        });
    });
}
exports.convertToRegularNfa = convertToRegularNfa;
